From: Mikulas Patocka <mpatocka@redhat.com>

Use i_size_read() instead of reading i_size.

If someone changes the size of the device simultaneously, i_size_read
is guaranteed to return a valid value (either the old one or the new one).

i_size can return some intermediate invalid value (on 32-bit computers
with 64-bit i_size, the reads to both halves of i_size can be interleaved
with updates to i_size, resulting in garbage being returned).

Cc: stable@kernel.org
Signed-off-by: Mikulas Patocka <mpatocka@redhat.com>
Signed-off-by: Alasdair G Kergon <agk@redhat.com>

---
drivers/md/dm-exception-store.h |    2 +-
 drivers/md/dm-exception-store.h |    2 +-
 drivers/md/dm-log.c             |    2 +-
 drivers/md/dm-table.c           |    3 ++-
 3 files changed, 4 insertions(+), 3 deletions(-)

Index: linux-2.6.30/drivers/md/dm-exception-store.h
===================================================================
--- linux-2.6.30.orig/drivers/md/dm-exception-store.h
+++ linux-2.6.30/drivers/md/dm-exception-store.h
@@ -156,7 +156,7 @@ static inline void dm_consecutive_chunk_
  */
 static inline sector_t get_dev_size(struct block_device *bdev)
 {
-	return bdev->bd_inode->i_size >> SECTOR_SHIFT;
+	return i_size_read(bdev->bd_inode) >> SECTOR_SHIFT;
 }
 
 static inline chunk_t sector_to_chunk(struct dm_exception_store *store,
Index: linux-2.6.30/drivers/md/dm-log.c
===================================================================
--- linux-2.6.30.orig/drivers/md/dm-log.c
+++ linux-2.6.30/drivers/md/dm-log.c
@@ -416,7 +416,7 @@ static int create_log_context(struct dm_
 				       bitset_size,
 				       ti->limits.logical_block_size);
 
-		if (buf_size > dev->bdev->bd_inode->i_size) {
+		if (buf_size > i_size_read(dev->bdev->bd_inode)) {
 			DMWARN("log device %s too small: need %llu bytes",
 				dev->name, (unsigned long long)buf_size);
 			kfree(lc);
Index: linux-2.6.30/drivers/md/dm-table.c
===================================================================
--- linux-2.6.30.orig/drivers/md/dm-table.c
+++ linux-2.6.30/drivers/md/dm-table.c
@@ -388,7 +388,8 @@ static void close_dev(struct dm_dev_inte
 static int check_device_area(struct dm_dev_internal *dd, sector_t start,
 			     sector_t len)
 {
-	sector_t dev_size = dd->dm_dev.bdev->bd_inode->i_size >> SECTOR_SHIFT;
+	sector_t dev_size = i_size_read(dd->dm_dev.bdev->bd_inode) >>
+			    SECTOR_SHIFT;
 
 	if (!dev_size)
 		return 1;
